#!/usr/bin/env python
# Copyright (C) 2012 Linaro
#
# This file is part of linaro-ci-dashboard.
#
# linaro-ci-dashboard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# linaro-ci-dashboard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with linaro-ci-dashboard.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.conf import settings
from xml.dom.minidom import parse, parseString
import jenkins


class JenkinsServer(models.Model):
    class Meta:
        app_label = 'jenkinsserver'

    url = models.CharField(max_length=255)
    username = models.CharField(max_length=100)
    password = models.CharField(max_length=100)

    def __init__(self, *args, **kwargs):
        super(JenkinsServer, self).__init__(*args, **kwargs)
        self.jenkins = jenkins.Jenkins(self.url,
                                       self.username.encode('utf-8'),
                                       self.password.encode('utf-8'))

    def import_jenkins_job(self, jobname):
        from frontend.models.loop import Loop
        jobname = jobname.replace(' ', '%20').strip()
        job = self.jenkins.get_job_info(jobname)
        loop = Loop()
        loop.name = job["name"]
        loop.server = self
        loop.save()

    def add_integration_loop(self, loop):
        jxml = self.create_job_xml(loop)
        self.create_job(loop.name, jxml) 

    def sync_jobs(self):
        """Sync jobs in dashboard DB with jobs in jenkins server.

        Removes all deleted jenkins jobs from dashboard db and
        adds all newly create jobs in dashboard db."""

        info = self.jenkins.get_info()
        jenkins_jobs = info['jobs']
        loops = self.loop_set.all()

        jenkins_jobs_names = [job['name'] for job in jenkins_jobs]
        loop_names = [loop.name for loop in loops]

        # Remove jobs present in our DB but not in Jenkins
        self.prune_jobs(self.unique_items(loop_names,
                                          jenkins_jobs_names))

        # Add jobs present in Jenkins but not our DB
        self.graft_jobs(self.unique_items(jenkins_jobs_names,
                                          loop_names))

    def prune_jobs(self, jobs):
        """Remove all items from 'jobs' list from the database."""
        for loop in self.loop_set.all():
            if loop.name in jobs:
                loop.delete()

    def graft_jobs(self, jobs):
        """Find all items from 'jobs' list in jenkins and add them
        to the database."""

        for job in jobs:
            self.import_jenkins_job(job)

    def unique_items(self, list_1, list_2):
        """Return a list of items that are present in list_1 but not list_2."""
        return [item for item in list_1 if item not in list_2]

    def create_job(self, jname, jxml):
        """ Creates a new job on jenkins if its  not already present """
        jname = jname.replace(' ', '%20').strip()
        if not self.jenkins.job_exists(jname):
            self.jenkins.create_job(jname, jxml)
            self.jenkins.enable_job(jname)

    def delete_job(self, jname):
        """ Deleted a job on jenkins if present """
        if self.jenkins.job_exists(jname):
            self.jenkins.delete_job(jname)
       
    def create_job_xml(self, loop):
        """ Prepares the config.xml to be used for new job"""

        dest_dir = 'dest_dir'
        env_cmds = "export USER=`whoami` ;\nexport HOME=/home/$USER"

        jbranch = jcmds = '' 
        if loop.branch:
            if not 'lp:' in loop.branch:
                jbranch = 'bzr branch lp:%s %s'% (loop.branch, dest_dir)
            else:
                jbranch = 'bzr branch %s %s'% (loop.branch, dest_dir)

        if loop.precommand:
            if "make syncdb" in loop.precommand:
                loop.precommand = 'yes "no" | make syncdb'
                

        rm_old_dest = 'rm -rf %s' % dest_dir
        dest_command = 'cur_dir=$WORKSPACE; cd ./%s' % dest_dir
        for cmd in [rm_old_dest, env_cmds, jbranch, dest_command, \
                        loop.precommand, loop.command]:
            if cmd: 
                jcmds = '\n'.join([jcmds, cmd])

        dest_command = "cd \"$cur_dir\""
        jcmds = '\n'.join([jcmds, dest_command]) 

        file = open(settings.JENKINS_JOB_CONFIG)
        jdata = file.read()
        file.close()

        doc = parseString(jdata)
        node = doc.getElementsByTagName('command')
        node[0].firstChild.nodeValue = jcmds
        jxml =  doc.toprettyxml()
        return jxml

    def schedule_job_build(self, jobname):
        """Trigger build job on jenkins."""
        self.jenkins.build_job(jobname)
