#!/usr/bin/env python
# Copyright (C) 2012 Linaro
#
# This file is part of linaro-ci-dashboard.
#
# linaro-ci-dashboard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# linaro-ci-dashboard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with linaro-ci-dashboard.  If not, see <http://www.gnu.org/licenses/>.
from django.conf import settings
from django.core.management.base import CommandError
from django.contrib.auth.models import User
from django.db import IntegrityError
from django.test import TestCase
from jenkinsserver.models.jenkins_server import JenkinsServer
from jenkins import JenkinsException
import jenkins
from frontend.models.loop import Loop
from mock import Mock, patch


class JenkinsServerTests(TestCase):

    def setUp(self):
        self.server = JenkinsServer.objects.create(
            url="http://127.0.0.1:%s/jenkins" % settings.JENKINS_PORT,
            username=settings.JENKINS_ADMIN_USER,
            password=settings.JENKINS_ADMIN_PASSWD)
        self.test_jenkins = jenkins.Jenkins(
            self.server.url,
            self.server.username.encode('utf-8'),
            self.server.password.encode('utf-8'))
        self.test_job_name = "test"
        if not self.test_jenkins.job_exists("test"):
            self.test_jenkins.create_job(self.test_job_name,
                                     jenkins.EMPTY_CONFIG_XML)

    def test_import_jenkins_job_existing(self):
        loop = Loop()
        loop.name = self.test_job_name
        loop.server = self.server
        loop.save()
        with self.assertRaises(IntegrityError):
            self.server.import_jenkins_job(self.test_job_name)

    def test_import_jenkins_job_non_existing(self):
        with self.assertRaises(JenkinsException):
            self.server.import_jenkins_job("non-existing-jenkins-job")

    def test_import_jenkins_job(self):
        loop = Loop()
        self.server.import_jenkins_job(self.test_job_name)
        loop = Loop.objects.get(name=self.test_job_name)
        self.assertIsNotNone(loop)

    def test_sync_jobs(self):
        self.server.sync_jobs()
        loops = [loop.name for loop in Loop.objects.all()]
        jenkins_jobs = [job["name"] for job in self.server.jenkins.get_jobs()]
        jenkins_jobs.sort()
        loops.sort()
        self.assertEquals(loops, jenkins_jobs)

    def test_unique_items(self):
        list_1 = ["1", "4", "5"]
        list_2 = ["1", "2", "5"]
        self.assertEquals(["4"], self.server.unique_items(list_1, list_2))
        self.assertEquals(["2"], self.server.unique_items(list_2, list_1))

    def test_graft_jobs(self):
        # This must be present on Jenkins server
        jobs = [self.test_job_name]
        self.server.graft_jobs(jobs)
        loops = [loop.name.encode("utf8") for loop in
                          Loop.objects.all()]
        loops.sort()
        self.assertEquals([self.test_job_name], loops)

    def test_prune_jobs(self):
        loop = Loop()
        loop.name = self.test_job_name
        loop.server = self.server
        loop.save()

        jobs = [self.test_job_name]
        self.server.prune_jobs(jobs)

        loops = [loop.name.encode("utf8") for loop in
                          self.server.loop_set.all()]
        self.assertNotIn(self.test_job_name, loops)

    def test_create_job_xml(self):
        loop = Loop()
        loop.name = 'sample-test'
        loop.server = self.server
        loop.branch = 'linaro-ci-dashboard'
        loop.precommand = 'cd %s' % loop.branch
        loop.command = 'make syncdb; make test'
        jbranch = 'bzr branch lp:%s dest_dir' % loop.branch
        dest_dir = 'dest_dir'
        env_cmds = "export USER=`whoami` ;\nexport HOME=/home/$USER"

        jbranch = jcommands = '' 
        if loop.branch:
            if not 'lp:' in loop.branch:
                jbranch = 'bzr branch lp:%s %s'% (loop.branch, dest_dir)
            else:
                jbranch = 'bzr branch %s %s'% (loop.branch, dest_dir)

        rm_old_dest = 'rm -rf %s' % dest_dir
        dest_command = 'cur_dir=$WORKSPACE; cd ./%s' % dest_dir
        for command in [rm_old_dest, env_cmds, jbranch, dest_command,\
                        loop.precommand, loop.command]:
            if command: 
                jcommands = '\n'.join([jcommands, command])

        dest_command = "cd &quot;$cur_dir&quot;"
        jcommands = '\n'.join([jcommands, dest_command]) 
        jxml = self.server.create_job_xml(loop)
        self.assertIn(jcommands, jxml)
        
    def test_create_new_job_delete_job(self):
        loop = Loop()
        loop.name = 'sample-test'
        loop.server = self.server
        loop.branch = 'linaro-ci-dashboard'
        loop.precommand = 'cd %s' % loop.branch
        loop.command = 'make syncdb; make test'
        jbranch = 'bzr branch lp:%s' % loop.branch
        jcommands = '\n'.join([jbranch,  loop.precommand, loop.command])
        jxml = self.server.create_job_xml(loop)
        self.server.create_job(loop.name, jxml)
        self.assertTrue(self.test_jenkins.job_exists(loop.name) == True)
        self.server.delete_job(loop.name)
        self.assertTrue(self.test_jenkins.job_exists(loop.name) == False)
